// Filename: LNODEFUN.CPP
// Purpose:  Shell file of functions to be used with the
//           demonstration program in LNODEDRV.CPP.  Some
//           functions are missing and will need to be
//           supplied by the student.

#include <iostream>
#include <iomanip>
using namespace std;

struct Node
{
    int data;
    Node* link;
};
typedef Node* NodePointer;


/**********************************************************************/

void DescribeProgram()
// Pre:  none
// Post: The program description has been displayed.
{
    cout << endl;
    cout << "This is a demonstration program used "
         << "to illustrate some operations on a "        << endl;
    cout << "sequence of linked nodes containing "
         << "integer data values.  It is best to "       << endl;
    cout << "run it with pencil and paper in hand "
         << "and to draw a pictorial representation "    << endl;
    cout << "of what the sequence of linked nodes "
         << "will look like after you take any "         << endl;
    cout << "particular action offered by the menu. "    << endl;
    cout << endl;
    cout << "You may display the data values in "
         << "the nodes at any time for purposes of "     << endl;
    cout << "comparison.  Each value is printed in six "
         << "spaces, with ten values per line. "         << endl;
    cout << endl;
    cout << "You begin with an empty list. "             << endl;
    cout << endl;
}

/**********************************************************************/

void DisplayMenu()
// Pre:  none
// Post: A menu with 16 options has been displayed.
{
    cout << endl;
    cout << endl;
    cout << "        Menu "                                   << endl;
    cout << endl;
    cout << " 1. Quit "                                       << endl;
    cout << " 2. Re-initialize sequence "
         <<      "(create a new empty sequence) "             << endl;
    cout << " 3. Display all node data values "
         <<      "in current sequence "                       << endl;
    cout << " 4. Build a new sequence of fixed size "         << endl;
    cout << " 5. Find the location of first node containing "
         <<      "a specified data value "                    << endl;
    cout << " 6. Find data value in kth node "                << endl;
    cout << " 7. Add a new node after first node containing "
         <<      "specified data value "                      << endl;
    cout << " 8. Add a new node after kth node "              << endl;
    cout << " 9. Add a new node before first node containing "
         <<      "specified data value "                      << endl;
    cout << "10. Add a new node before kth node "             << endl;
    cout << "11. Delete first node containing a "
         <<      "specified data value "                      << endl;
    cout << "12. Delete kth node "                            << endl;
    cout << endl;
}

/**********************************************************************/

void GetMenuChoice(/* out */ int& menuChoice)
// Pre:  none
// Post: "menuChoice" contains an integer entered by the user
//       from the keyboard
{
    cout << "Enter menu choice here: ";
    cin >> menuChoice;  cin.ignore(80, '\n');
    cout << endl;
}

/**********************************************************************/

void InitializeSequence(/* inout */ NodePointer& head)
// Pre:  none
// Post: "head" has been set to NULL.
{
    head = NULL;
}

/**********************************************************************/

bool EmptySequence(/* in */ NodePointer head)
// Pre:  "head" has been initialized.
// Post: The function value is true if the sequence is empty,
//       and false otherwise.
{
    return (head == NULL);
}

/**********************************************************************/

void ReclaimStorage(/* in */ NodePointer head)
// Pre:  "head" has been initialized.
// Post: All storage (if any) in the node sequence pointed to by
//       "head" has been returned to the free store.
{
    NodePointer current;

    while (head != NULL)
    {
        current = head;
        head = head->link;
        delete current;
    }
}

/**********************************************************************/

void BuildSequenceFixedSize(/* inout */ NodePointer& head)
// Pre:  none
// Post: The user has entered the number of nodes to be in a
//       newly constructed sequence, as well as the values to
//       be inserted into the sequence, the sequence has been
//       constructed, and "head" points at the first node.
//       All storage in any sequence previously pointed to by
//       "head" has been returned to the free store.
{
    if (!EmptySequence(head))
    {
        ReclaimStorage(head);
        InitializeSequence(head);
    }

	//Your codes goes here

}

/**********************************************************************/

void PrintNodeValues(/* inout */ NodePointer& head)
// Pre:  "head" has been initialized.
// Post: If the sequence is empty, a message to that effect has
//       been output.  Otherwise, all values in the sequence
//       pointed to by "head" have been displayed, ten to a line,
//       with each one in a field width of six spaces.
{
    // Function body goes here

}

/**********************************************************************/

void FindNodeDataValue(/* inout */ NodePointer   head,
                       /* in */    DataType      targetValue,
                       /* out */   bool&         found,
                       /* out */   NodePointer&  targetPointer,
                       /* out */   int&          targetPosition)
// Pre:  "head" and "targetValue" have been initialized.
// Post: If "targetValue" is in the sequence, "found" is true and:
//       - "targetPointer" points to the first node of the sequence
//         containing the "targetValue"
//       - "targetPosition" contains the number of the first node
//         in the sequence containing the "targetValue"
//       Otherwise, "found" is false and both "targetPointer" and
//       "targetPosition" are undefined.
{
    

    NodePointer current = head;
    int nodeCounter = 1;

    found = false;

    while (!found  &&  current != NULL)
    {
        // Your codes goes here

    }

    if (found)
    {
        targetPointer  = current;
        targetPosition = nodeCounter;
    }


}

/**********************************************************************/

void FindDataValueAndPrevious(/* inout */ NodePointer&  head,
                              /* in */    DataType      targetValue,
                              /* out */   bool&         found,
                              /* out */   NodePointer&  targetPointer,
                              /* out */   NodePointer&  previousPointer)
// Pre:  "head" and "targetValue" have been initialized.
// Post: If "targetValue" is in the sequence, "found" is true and
//       "targetPointer" points at the first node of the sequence
//       containing "targetValue", with "previousPointer" pointing
//       at the previous node, unless "targetPointer" is pointing at
//       the first node, in which case the value of "previousPointer"
//       is NULL.  Otherwise, "found" is false and the values of both
//       "targetPointer" and "previousPointer" are undefined.
{
    NodePointer current = head;
    NodePointer previous = NULL;

    found = false;

    while (!found  &&  current != NULL)
    {
        // Your codes goes here

    }

    if (found)
    {
        targetPointer   = current;
        previousPointer = previous;
    }


}

/**********************************************************************/

void FindNodeK(/* inout */ NodePointer&  head,
               /* in */    int           k,
               /* out */   bool&         found,
               /* out */   NodePointer&  targetPointer)
// Pre:  "head" and "k" have been initialized.
// Post: If the sequence contains k or more nodes, "found" is true
//       and "targetPointer" points to node k.  Otherwise, "found"
//       is false and "targetPointer" is undefined.
{
	    NodePointer current = head;
    int nodeCounter = 1;

	found = false;

	while(current->link != NULL && nodeCounter != k)
	{
        // Your codes goes here

	}

	if (nodeCounter == k) {found = true; targetPointer = current;}



}

/******************************************************************/

void FindNodeKAndPrevious(/* inout */ NodePointer&  head,
                          /* in */    int           k,
                          /* out */   bool&         found,
                          /* out */   NodePointer&  targetPointer,
                          /* out */   NodePointer&  previousPointer)
// Pre:  "head" and "k" have been initialized.
// Post: If the sequence has k or more nodes, "found" is true and
//       "targetPointer" points at node k of the sequence, with
//       "previousPointer" pointing at the previous node, unless
//       node k is the first node, in which case the value of
//       "previousPointer" is NULL.  Otherwise, "found" is false
//       and the values of both "targetPointer" and "previousPointer"
//       are undefined.
{
    	NodePointer current = head;
    NodePointer previous = NULL;
	found = false;

    int nodeCounter = 1;

    while (current->link != NULL && nodeCounter != k)
    {
        // Your codes goes here
    }

	if (nodeCounter == k) 
	{
		targetPointer = current;
		previousPointer = previous;
		found = true;
	}


}
/*******************************************************************/

void FindDataValues(/* inout */ NodePointer& head)
// Pre:  "head" has been initialized.
// Post: The user has entered zero or more integer values and
//       in each case a message has been displayed, indicating
//       either the first position in the sequence where that
//       value was located or that the value was not found in
//       in the current sequence.
{
    DataType targetValue;
    NodePointer targetPointer;
    int targetPosition;
    bool found;

    cout << "Enter the data value to look for, "
         << "or enter 0 to quit. ";
    cin >> targetValue;  cin.ignore(80, '\n');

    while (targetValue!=0)
    {
        cout << endl;

        FindNodeDataValue(head, targetValue, found,
                          targetPointer, targetPosition);

        // Your codes goes here
    }
    cout << endl << endl;

    cin.clear();


}

/**********************************************************************/

void FindPositions(/* inout */ NodePointer& head)
// Pre:  "head" has been initialized.
// Post: The user has entered zero or more integer position
//       numbers and in each case a message has been displayed,
//       indicating either the value found in that position or
//       that position was not found in the current sequence.
{
    // Function body goes here
    // You need call FindNodeK(head, targetPosition, found, targetPointer);
    // in this function
}

/**********************************************************************/

void InsertAfterNodeWithDataValue(/* inout */ NodePointer& head)
// Pre:  "head" has been initialized.
// Post: The user has entered zero or more data value pairs, and
//       in each case the second value in the pair has been inserted
//       into the sequence after the first occurrence of the first
//       value in the sequence, or a message has been displayed
//       indicating that the first value of the pair was not found
//       in the current sequence.
{
    DataType targetValue;
    NodePointer newNode;
    NodePointer targetPointer;
    int targetPosition;
    bool found;

    cout << "\nEnter data value after which to insert, "
         << "or enter 0 to quite. ";
    cin >> targetValue;  cin.ignore(80, '\n');

    while (targetValue != 0)
    {
        cout << endl;

        FindNodeDataValue(head, targetValue, found,
                          targetPointer, targetPosition);
        if (found)
        {
            // Your codes goes here
        }
        else
        {
            cout << endl;
            cout << "There is no node containing the value "
                 << targetValue << "."                      << endl;
        }

        cout << endl;
        cout << "Enter another data value after which to insert, "
             << "or enter 0 to quite. ";
        cin >> targetValue;  cin.ignore(80, '\n');
    }
    cout << endl << endl;

    cin.clear();
}

/************************************************************************/

void InsertAfterNodeK(/* inout */ NodePointer& head)
// Pre:  "head" has been initialized.
// Post: The user has entered zero or more position/value pairs,
//       and in each case the value in the pair has been inserted
//       into the sequence after the given position, or a message
//       has been displayed indicating that the given position was
//       not found in the current sequence.
{
    // Function body goes here

}
/********************************************************************/

void InsertBeforeNodeWithDataValue(/* inout */ NodePointer& head)
// Pre:  "head" has been initialized.
// Post: The user has entered zero or more data value pairs, and
//       in each case the second value in the pair has been inserted
//       into the sequence before the first occurrence of the first
//       value in the sequence, or a message has been displayed
//       indicating that the first value of the pair was not found
//       in the current sequence.
{
    NodePointer newNode;
    NodePointer targetPointer;
    int targetPosition;
    bool found;

    cout << "\nEnter the position number after which to insert, "
         << "or enter 0 to quite. ";
    cin >> targetPosition;  cin.ignore(80, '\n');

    while (targetPosition != 0)
    {
        cout << endl;

        FindNodeK(head, targetPosition, found, targetPointer);
        if (found)
        {
            // Your codes goes here
        }
        else
        {
            cout << endl;
            cout << "There is no node in the position of "
                 << targetPosition << "."                      << endl;
        }

        cout << endl;
		cout << "\nEnter the position number after which to insert, "
         << "or enter 0 to quite. ";
        cin >> targetPosition;  cin.ignore(80, '\n');
    }
    cout << endl << endl;

    cin.clear();


}

/**********************************************************************/

void InsertBeforeNodeK(/* inout */ NodePointer& head)
// Pre:  "head" has been initialized.
// Post: The user has entered zero or more position/value pairs,
//       and in each case the value in the pair has been inserted
//       into the sequence before the given position, or a message
//       has been displayed indicating that the given position was
//       not found in the current sequence.
{
       // Function body goes here
       // You need call FindNodeKAndPrevious(head,targetPosition,found,targetPointer,previousPointer);
       // in this function
}
/*************************************************************************/

void DeleteNodeWithDataValue(/* inout */ NodePointer& head)
// Pre:  "head" has been initialized.
// Post: The user has entered zero or more data values and in
//       each case the first node of the sequence containing
//       that value has been deleted from the sequence and its
//       storage has been returned to the free store, or a
//       message has been displayed indicating that the value
//       was not found in the current sequence.
{
       // Function body goes here
	 DataType targetValue;
    NodePointer targetPointer;
    NodePointer previousPointer;
    bool found;

    cout << endl;
    cout << "Enter data value to delete, or "
         << "or enter 0 to quite. ";
    cin >> targetValue;  cin.ignore(80, '\n');

    while (targetValue != 0)
    {
        cout << endl;

        FindDataValueAndPrevious(head, targetValue, found,
                                 targetPointer, previousPointer);

        if (found)
        {
            // Your codes goes here
        }
        else
        {
            cout << endl;
            cout << "There is no node containing the value "
                 << targetValue << ".";
        }

        cout << endl;
        cout << "Enter another data value to delete, "
             << "or enter 0 to quite. ";
        cin >> targetValue;  cin.ignore(80, '\n');
    }

    cin.clear();

}

/*************************************************************************/

void DeleteNodeK(/* inout */ NodePointer& head)
// Pre:  "head" has been initialized.
// Post: The user has entered zero or more node positions and
//       in each case the node of the sequence having the given
//       position has been deleted from the sequence and its
//       storage has been returned to the free store, or a
//       message has been displayed indicating that the given
//       position was not found in the current sequence.
{
    	NodePointer targetPointer;
    NodePointer previousPointer;
	int targetPosition;

    bool found;

    cout << endl;
    cout << "Enter the position to delete, or "
            "or enter 0 to quite. ";
    cin >> targetPosition;  cin.ignore(80, '\n');

    while (targetPosition != 0)
    {
        cout << endl;

        
		FindNodeKAndPrevious(head,targetPosition,found,targetPointer,previousPointer);

        if (found)
        {
            // Your codes goes here
        }
        else
        {
            cout << endl;
            cout << "There is no node in the position of "
                 << targetPosition << ".";
        }

        cout << endl;
		cout << "Enter the position to delete, or "
			 << "or enter 0 to quite. ";
		cin >> targetPosition;  cin.ignore(80, '\n');
    }

    cin.clear();


}

