// Filename: TIME2.CPP
// Purpose:  Implementation file corresponding to TIME1.H.
//           This version adds two constructors to the version in TIME1.CPP.

#include <iostream>
using namespace std;

#include "TIME.H"


// Private data members of the Time class:
//     int hours;
//     int minutes;
//     int seconds;


//******************************************************************
Time::Time()
// Default constructor
// Pre:  none
// Post: Class object is constructed and the time is 00:00:00.
//       That is, hours, minutes and seconds all have value 0.
{
    hours = 0;
    minutes = 0;
    seconds = 0;
}


//******************************************************************
Time::Time(/* in */ int hoursInitial,
           /* in */ int minutesInitial,
           /* in */ int secondsInitial)
// Constructor
// Pre:      0 <= hoursInitial   <= 23
//           0 <= minutesInitial <= 59
//       and 0 <= secondsInitial <= 59
// Post: Class object is constructed.
//       The time is set according to the incoming parameters.
{
    hours = hoursInitial;
    minutes = minutesInitial;
    seconds = secondsInitial;
}


//******************************************************************
void Time::Set(/* in */ int hoursNew,
               /* in */ int minutesNew,
               /* in */ int secondsNew)
// Pre:  0 <= hoursNew <= 23
//       0 <= minutesNew <= 59
//       0 <= secondsNew <= 59
// Post: Time is set according to the incoming parameters.
{
    hours = hoursNew;
    minutes = minutesNew;
    seconds = secondsNew;
}



//******************************************************************
void Time::Increment()
// Pre:  none
// Post: Time has been advanced by one second, with
//       23:59:59 wrapping around to 00:00:00.
{
    seconds++;
    if (seconds > 59)
    {
        seconds = 0;
        minutes++;
        if (minutes > 59)
        {
            minutes = 0;
            hours++;
            if (hours > 23)
                hours = 0;
        }
    }
}


//******************************************************************
void Time::Display() const
// Pre:  none
// Post: Time has been output in the form HH:MM:SS.
{
    if (hours < 10)
        cout << '0';
    cout << hours << ':';
    if (minutes < 10)
        cout << '0';
    cout << minutes << ':';
    if (seconds < 10)
        cout << '0';
    cout << seconds;
}
