// Filename: TIME4.CPP
// Purpose:  Implementation file corresponding to TIME4.H.
//           This version adds three overloaded operators and default
//           parameters in order to combine constructor definitions.

#include <iostream>
using namespace std;

#include "TIME4.H"


// Private data members of the Time class:
//     int hours;
//     int minutes;
//     int seconds;


// This is a "four-in-one" constructor, which may take zero, one,
// two, or three parameters.
//******************************************************************
Time::Time(/* in */ int hoursInitial,
           /* in */ int minutesInitial,
           /* in */ int secondsInitial)
// Constructor
// Pre:  0 <= hoursInitial   <= 23  and
///      0 <= minutesInitial <= 59  and
//       0 <= secondsInitial <= 59
// Post: Class object is constructed
//       and the time is set according to the incoming parameters.
{
    hours = hoursInitial;
    minutes = minutesInitial;
    seconds = secondsInitial;
}


// The following five "interface functions" are exactly the same as
// the corresponding ones in TIME3.CPP.  For brevity we have omitted
// comments and pre/post-conditions, which are also the same, as are
// the function implementations.


//******************************************************************
void Time::Set(int hoursNew, int minutesNew, int secondsNew)
{
    hours = hoursNew;
    minutes = minutesNew;
    seconds = secondsNew;
}


//******************************************************************
void Time::Increment()
{
    seconds++;
    if (seconds > 59)
    {
        seconds = 0;
        minutes++;
        if (minutes > 59)
        {
            minutes = 0;
            hours++;
            if (hours > 23)
                hours = 0;
        }
    }
}

//******************************************************************
int Time::Hours()   const { return hours; }
int Time::Minutes() const { return minutes; }
int Time::Seconds() const { return seconds; }



// The following functions are the definitions of the four overloaded
//  operators supplied by this implementation of the Time class.

//******************************************************************
Time Time::operator+(/* in */ const Time& otherTime) const
// Pre:  self and otherTime have been initialized.
// Post: Value returned is equivalent to self advanced by otherTime.
{
    int hrs;
    int mins;
    int secs;

    Time t;

    secs = seconds + otherTime.seconds;
    t.seconds = secs % 60;
    mins = minutes + otherTime.minutes + secs / 60;
    t.minutes = mins % 60;
    hrs = hours + otherTime.hours + mins / 60;
    t.hours = hrs % 24;

    return t;
}



//******************************************************************
bool Time::operator==(/* in */ const Time& otherTime) const
// Pre:  none
// Post: Returns true if self equals otherTime and false otherwise.
{
    return (hours == otherTime.hours      &&
            minutes == otherTime.minutes  &&
            seconds == otherTime.seconds);
}


//******************************************************************
istream& operator>>(/* inout */ istream& inputStream,
                    /* out */   Time& t)
// Pre:  The input stream inputStream is open.
//       Time values in the input stream have the form HH:MM:SS.
// Post: Time t has been read in.
//       inputStream is still open.
{
    char ch;

    inputStream >> t.hours   >> ch
                >> t.minutes >> ch
                >> t.seconds;

    return inputStream;
}


//******************************************************************
ostream& operator<<(/* inout */ ostream& outputStream,
                    /* in */    const Time& t)
// Pre:  The output stream outputStream is open.
// Post: Time t has been output in the form HH:MM:SS.
//       outputStream is still open.
{
    if (t.hours < 10)
        outputStream << '0';
    outputStream << t.hours << ':';
    if (t.minutes < 10)
        outputStream << '0';
    outputStream << t.minutes << ':';
    if (t.seconds < 10)
        outputStream << '0';
    outputStream << t.seconds;

    return outputStream;
}
