// Filename: TIME4.H
// Purpose:  Specification file corresponding to TIME4.CPP.
//           This version adds three overloaded operators and default
//           parameters in order to combine constructor definitions.


#ifndef TIME4_H
#define TIME4_H


class Time
{
    // The following two functions are "overloaded operators",
    // implemented as friend functions.

    friend istream& operator>>(/* inout */ istream& inputStream,
                               /* out */    Time& t);
    // Pre:  The output stream outputStream is open.
    // Post: Time t has been output in the form HH:MM:SS.
    //       outputStream is still open.

    friend ostream& operator<<(/* inout */ ostream& outputStream,
                               /* in */    const Time& t);
    // Pre:  The output stream outputStream is open.
    // Post: Time t has been output in the form HH:MM:SS.
    //       outputStream is still open.

public:

    // A new "four-in-one" constructor with "default parameter values".
    Time(/* in */ int hoursInitial   = 0,
         /* in */ int minutesInitial = 0,
         /* in */ int secondsInitial = 0);

    // The following five "interface functions" are exactly the
    // same as those in TIME3.H.  For brevity we have omitted the
    // comments and pre/post-conditions, which are also the same.
    // Note that the constructors and Display are missing.
    // The constructors have been replaced by the new one above.
    // Display will now be implemented as an overloaded operator,
    // as will the (unique) equality operation and an add operation.
    void Set(int hoursNew, int minutesNew, int secondsNew);
    void Increment();
    int Hours() const;
    int Minutes() const;
    int Seconds() const;


    // The following two functions are also "overloaded operators",
    // but are member functions.

    Time operator+(/* in */ const Time& otherTime) const;
    // Pre:  self and otherTime have been initialized.
    // Post: Value returned is equivalent to self advanced by otherTime.

    bool operator==(/* in */ const Time& otherTime) const;
    // Pre:  none
    // Post: Returns true if self equals otherTime and false otherwise.


private:

    int hours;
    int minutes;
    int seconds;
};

#endif
